function [error] = readAsciiData( hMainFig, filename, type)
% Function to read Bucket data in ASCII format. 
%
% Input
% -----
% hMainFig ........... Handel to the main user interface
% filename ........... Data filename
% type ............... Can read 3 types of data : WS, EXT, TSG
%
% Output
% ------
% error .............. 1: OK - -1 : an error occured

% Get the data from the application GUI
% -------------------------------------
tsg = getappdata( hMainFig, 'tsg_data');

% Get the fieldnames of tsg structure
% -----------------------------------
tsgNames     = fieldnames(tsg);
nbFieldNames = length( tsgNames );

% Get NO_CONTROL code value
% -------------------------
NO_CONTROL = get(tsg.qc.hash, 'NO_CONTROL', 'code');

% Display read file info on console
% ---------------------------------
fprintf('\nREAD_ASCII_FILE\n'); tic;

% Open the file
% -------------
fid = fopen( filename, 'rt' );

% Check file
% -----------
if fid == -1
  msg_error = ['TSG_GOSUD file_lecture : Open file error : ' filename];
  warndlg( msg_error, 'ASCII error dialog');
  sprintf('...cannot locate %s\n', filename);
  error = -1;
  return;
end

% Display more info about read file on console
% --------------------------------------------
fprintf('...reading %s : ', filename);

% Read the header till the header line has been read
% --------------------------------------------------
OK = 0;
while ~OK
  
  % Read every line
  % ---------------
  line = fgetl( fid );
  c = textscan( line, '%s');
  
  switch char( c{1}(1) )

    case '%HEADER'

      % Read the header then quit the while loop
      % ----------------------------------------
      header = c{1}(2:end);
      nHeader = length( header );
      OK = 1;

    otherwise

      % Get the parameter Name (Delete '%')
      % ----------------------------------
      Para = char( strtok(c{1}(1), '%') );
      
      % Read the parameter value
      % ------------------------
      ind = strmatch( Para, tsgNames);
      if ~isempty( ind )
        tsg.(Para) = char( c{1}(2) );
      end
      
      if strcmp( Para, 'SSPS_WS_ANALDATE');
        tsg.(Para) = [];
      end
      
  end
end

% Builld the format depending on the header parameters
% 1 - Decimate the HEADER - The 7th first parameters are always
%     %HEADER YEAR MNTH DAYX hh mm ss
% 2 - The 6 Date and time parametes are read in %d
% -------------------------------------------------------------
format = '%d %d %d %d %d %d';
for i = 7 : nHeader
  if isempty( strfind( char(header(i)), 'TYPE') );
    format = [format ' %f'];
  else
    format = [format ' %s'];
  end
end

% Read the data in a cell
% -----------------------
cellData = textscan( fid, format );

% Convert cell to a structure
% ---------------------------
s = cell2struct(cellData, header, 2);

clear cellData

% Test if the fiels _TYPE and _QC exists for the variables
% --------------------------------------------------------
PARA = {'SSPS_WS'; 'SSPS_EXT'; 'SSTP_EXT' };
for i = 1:3
  para = PARA{i};
  
  for j = 7 : nHeader
    
    if ~isempty( strfind( char(header(j)), para ))
      if isempty( strfind( char(header(j)), [para '_TYPE']))
        label = 'UNKN';
        s.([para '_TYPE']) = label( ones(length(s.(para)),1), :);
        header = [header; [para '_TYPE']];
      end
      
      if isempty( strfind( char(header(j)), [para '_QC']))
        s.([para '_QC']) = NO_CONTROL * int8(ones( size( s.(para) )));
        header = [header; [para '_QC']];
      end
    end
    
  end
end

% Date (y m d h m s) in the first 6 elements in data
% --------------------------------------------------
yy = double( s.(char(header(1))) );
mm = double( s.(char(header(2))) );
dd = double( s.(char(header(3))) );
hh = double( s.(char(header(4))) );
mi = double( s.(char(header(5))) );
ss = double( s.(char(header(6))) );

switch type

  case 'WS'

    replace = 1;
    if ~isempty( tsg.DAYD_WS )

      errmsg = {'Water samples already loaded '; ' '; ...
        'Do you want to replace the existing samples ?'};
      button = questdlg(errmsg, 'Sample','Yes', 'No', 'No');

      switch button
        case 'Yes'

          replace = 1;
          tsg.DAYD_WS      = datenum(yy, mm, dd, hh, mi, ss);
          tsg.SSPS_WS      = [];
          tsg.SSPS_WS_QC   = [];
          tsg.SSPS_WS_TYPE = [];

          tsg.report.([ lower(type) 'file']) = filename;

        case 'No'
          replace = 0;
      end
      
    else
      tsg.DAYD_WS      = datenum(yy, mm, dd, hh, mi, ss);
    end

  case 'EXT'

    replace = 1;
    if ~isempty( tsg.DAYD_EXT )

      errmsg = {'Water samples already loaded '; ' '; ...
        'Do you want to replace the existing samples ?'};
      button = questdlg(errmsg, 'Sample','Yes', 'No', 'No');

      switch button
        case 'Yes'

          replace = 1;
          tsg.DAYD_EXT  = datenum(yy, mm, dd, hh, mi, ss);
          tsg.SSPS_EXT      = [];
          tsg.SSPS_EXT_QC   = [];
          tsg.SSPS_EXT_TYPE = [];
          tsg.SSTP_EXT      = [];
          tsg.SSTP_EXT_QC   = [];
          tsg.SSTP_EXT_TYPE = [];

          tsg.report.([ lower(type) 'file']) = filename;

        case 'No'
          replace = 0;
      end
    else
      tsg.DAYD_EXT  = datenum(yy, mm, dd, hh, mi, ss);

    end

  case 'TSG'

    replace = 1;
    tsg.DAYD = datenum(yy, mm, dd, hh, mi, ss);
    tsg.report.tsgfile = filename;

  otherwise
    msg('function ''readTsgDataAscii'' - Bad TYPE', 'Modal' );
end


% Loop over the header. Skip the 6 first elements
% -----------------------------------------------
if replace
  nHeader = length( header );
  for i = 7 : nHeader


    % QC should be converted in int8
    % TYPE  must be converted in string4
    % ----------------------------------
    if ~isempty( strfind(char(header(i)), '_QC'))
      tsg.(char(header(i))) = int8(s.(char(header(i))));
    elseif ~isempty( strfind(char(header(i)), '_TYPE'))
      tsg.(char(header(i))) = char( s.(char(header(i))) );
    else
      tsg.(char(header(i))) = s.(char(header(i)));
    end

  end

end

if strcmp( type ,'TSG' )

  % populate tsg.file structure
  % ---------------------------
  [tsg.file.pathstr, tsg.file.name, tsg.file.ext, tsg.file.versn] = ...
                                                           fileparts(filename);

  % Perform somme automatic tests
  % -----------------------------
  automaticQC( hMainFig );

  tsg.file.type = 'ASCII';

end

% Save the data in the application GUI
% ------------------------------------
setappdata( hMainFig, 'tsg_data', tsg );

% Close the file
% --------------
fclose( fid );

% Display time to read file on console
% ------------------------------------
t = toc; fprintf('...done (%6.2f sec).\n\n',t);

% Everything OK
% -------------
error = 1;

end
